@@ -30,12 +30,17 @@ from raven.conf import defaults
 from raven.conf.remote import RemoteConfig
 from raven.context import Context
 from raven.exceptions import APIError, RateLimited
-from raven.utils import six, json, get_versions, get_auth_header, merge_dicts
+from raven.utils import json, get_versions, get_auth_header, merge_dicts
+from raven._compat import text_type, iteritems
 from raven.utils.encoding import to_unicode
 from raven.utils.serializer import transform
 from raven.utils.stacks import get_stack_info, iter_stack_frames, get_culprit
 from raven.transport.registry import TransportRegistry, default_transports
 
+# enforce imports to avoid obscure stacktraces with MemoryError
+import raven.events  # NOQA
+
+
 __all__ = ('Client',)
 
 __excepthook__ = None
@@ -128,8 +133,6 @@ class Client(object):
 
         o = options
 
-        self.configure_logging()
-
         self.raise_send_errors = raise_send_errors
 
         # configure loggers first
@@ -145,7 +148,7 @@ class Client(object):
 
         self.include_paths = set(o.get('include_paths') or [])
         self.exclude_paths = set(o.get('exclude_paths') or [])
-        self.name = six.text_type(o.get('name') or o.get('machine') or defaults.NAME)
+        self.name = text_type(o.get('name') or o.get('machine') or defaults.NAME)
         self.auto_log_stacks = bool(
             o.get('auto_log_stacks') or defaults.AUTO_LOG_STACKS)
         self.capture_locals = bool(
@@ -220,14 +223,6 @@ class Client(object):
     def register_scheme(cls, scheme, transport_class):
         cls._registry.register_scheme(scheme, transport_class)
 
-    def configure_logging(self):
-        for name in ('raven', 'sentry'):
-            logger = logging.getLogger(name)
-            if logger.handlers:
-                continue
-            logger.addHandler(logging.StreamHandler())
-            logger.setLevel(logging.INFO)
-
     def get_processors(self):
         for processor in self.processors:
             yield self.module_cache[processor](self)
@@ -252,7 +247,8 @@ class Client(object):
         >>> result = client.capture(**kwargs)
         >>> ident = client.get_ident(result)
         """
-        warnings.warn('Client.get_ident is deprecated. The event ID is now returned as the result of capture.',
+        warnings.warn('Client.get_ident is deprecated. The event ID is now '
+                      'returned as the result of capture.',
                       DeprecationWarning)
         return result
 
@@ -276,6 +272,32 @@ class Client(object):
             return url
         return '%s:%s' % (scheme, url)
 
+    def _get_exception_key(self, exc_info):
+        # On certain celery versions the tb_frame attribute might
+        # not exist or be `None`.
+        code_id = 0
+        last_id = 0
+        try:
+            code_id = id(exc_info[2] and exc_info[2].tb_frame.f_code)
+            last_id = exc_info[2] and exc_info[2].tb_lasti or 0
+        except (AttributeError, IndexError):
+            pass
+        return (
+            exc_info[0],
+            id(exc_info[1]),
+            code_id,
+            id(exc_info[2]),
+            last_id,
+        )
+
+    def skip_error_for_logging(self, exc_info):
+        key = self._get_exception_key(exc_info)
+        return key in self.context.exceptions_to_skip
+
+    def record_exception_seen(self, exc_info):
+        key = self._get_exception_key(exc_info)
+        self.context.exceptions_to_skip.add(key)
+
     def build_msg(self, event_type, data=None, date=None,
                   time_spent=None, extra=None, stack=None, public_key=None,
                   tags=None, fingerprint=None, **kwargs):
@@ -306,7 +328,7 @@ class Client(object):
         if data.get('culprit'):
             culprit = data['culprit']
 
-        for k, v in six.iteritems(result):
+        for k, v in iteritems(result):
             if k not in data:
                 data[k] = v
 
@@ -394,11 +416,11 @@ class Client(object):
             data['message'] = kwargs.get('message', handler.to_string(data))
 
         # tags should only be key=>u'value'
-        for key, value in six.iteritems(data['tags']):
+        for key, value in iteritems(data['tags']):
             data['tags'][key] = to_unicode(value)
 
         # extra data can be any arbitrary value
-        for k, v in six.iteritems(data['extra']):
+        for k, v in iteritems(data['extra']):
             data['extra'][k] = self.transform(v)
 
         # It's important date is added **after** we serialize
@@ -524,6 +546,12 @@ class Client(object):
         if not self.is_enabled():
             return
 
+        exc_info = kwargs.get('exc_info')
+        if exc_info is not None:
+            if self.skip_error_for_logging(exc_info):
+                return
+            self.record_exception_seen(exc_info)
+
         data = self.build_msg(
             event_type, data, date, time_spent, extra, stack, tags=tags,
             **kwargs)
@@ -556,7 +584,8 @@ class Client(object):
             if isinstance(exc, RateLimited):
                 retry_after = exc.retry_after
             self.error_logger.error(
-                'Sentry responded with an API error: %s(%s)', type(exc).__name__, exc.message)
+                'Sentry responded with an API error: %s(%s)',
+                type(exc).__name__, exc.message)
         else:
             self.error_logger.error(
                 'Sentry responded with an error: %s (url: %s)\n%s',
@@ -577,10 +606,10 @@ class Client(object):
         if 'exception' in data and 'stacktrace' in data['exception']['values'][0]:
             # try to reconstruct a reasonable version of the exception
             for frame in data['exception']['values'][0]['stacktrace']['frames']:
-                output.append('  File "%(filename)s", line %(lineno)s, in %(function)s' % {
-                    'filename': frame['filename'],
+                output.append('  File "%(fn)s", line %(lineno)s, in %(func)s' % {
+                    'fn': frame['filename'],
                     'lineno': frame['lineno'],
-                    'function': frame['function'],
+                    'func': frame['function'],
                 })
 
         self.uncaught_logger.error(output)
@@ -626,8 +655,7 @@ class Client(object):
     def send_encoded(self, message, auth_header=None, **kwargs):
         """
         Given an already serialized message, signs the message and passes the
-        payload off to ``send_remote`` for each server specified in the servers
-        configuration.
+        payload off to ``send_remote``.
         """
         client_string = 'raven-python/%s' % (raven.VERSION,)
 
@@ -694,6 +722,8 @@ class Client(object):
 
         ``kwargs`` are passed through to ``.capture``.
         """
+        if exc_info is None or exc_info is True:
+            exc_info = sys.exc_info()
         return self.capture(
             'raven.events.Exception', exc_info=exc_info, **kwargs)
 
